@@ -4,10 +4,13 @@
 Test cases related to SAX I/O
 """
 
-import unittest
-from StringIO import StringIO
+import unittest, sys, os.path
 
-from common_imports import HelperTestCase, doctest
+this_dir = os.path.dirname(__file__)
+if this_dir not in sys.path:
+    sys.path.insert(0, this_dir) # needed for Py3
+
+from common_imports import HelperTestCase, doctest, make_doctest, BytesIO, _bytes
 from lxml import sax
 from xml.dom import pulldom
 
@@ -16,19 +19,43 @@ class ETreeSaxTestCase(HelperTestCase):
     def test_etree_sax_simple(self):
         tree = self.parse('<a>ab<b/>ba</a>')
         xml_out = self._saxify_serialize(tree)
-        self.assertEquals('<a>ab<b/>ba</a>',
+        self.assertEqual(_bytes('<a>ab<b/>ba</a>'),
                           xml_out)
 
     def test_etree_sax_double(self):
         tree = self.parse('<a>ab<b>bb</b>ba</a>')
         xml_out = self._saxify_serialize(tree)
-        self.assertEquals('<a>ab<b>bb</b>ba</a>',
+        self.assertEqual(_bytes('<a>ab<b>bb</b>ba</a>'),
+                          xml_out)
+
+    def test_etree_sax_comment(self):
+        tree = self.parse('<a>ab<!-- TEST -->ba</a>')
+        xml_out = self._saxify_serialize(tree)
+        self.assertEqual(_bytes('<a>abba</a>'),
+                          xml_out)
+
+    def test_etree_sax_pi(self):
+        tree = self.parse('<a>ab<?this and that?>ba</a>')
+        xml_out = self._saxify_serialize(tree)
+        self.assertEqual(_bytes('<a>ab<?this and that?>ba</a>'),
+                          xml_out)
+
+    def test_etree_sax_comment_root(self):
+        tree = self.parse('<!-- TEST --><a>ab</a>')
+        xml_out = self._saxify_serialize(tree)
+        self.assertEqual(_bytes('<a>ab</a>'),
+                          xml_out)
+
+    def test_etree_sax_pi_root(self):
+        tree = self.parse('<?this and that?><a>ab</a>')
+        xml_out = self._saxify_serialize(tree)
+        self.assertEqual(_bytes('<?this and that?><a>ab</a>'),
                           xml_out)
 
     def test_etree_sax_attributes(self):
         tree = self.parse('<a aa="5">ab<b b="5"/>ba</a>')
         xml_out = self._saxify_serialize(tree)
-        self.assertEquals('<a aa="5">ab<b b="5"/>ba</a>',
+        self.assertEqual(_bytes('<a aa="5">ab<b b="5"/>ba</a>'),
                           xml_out)
 
     def test_etree_sax_ns1(self):
@@ -74,11 +101,11 @@ class ETreeSaxTestCase(HelperTestCase):
         b = a[0]
 
         xml_out = self._saxify_serialize(a)
-        self.assertEquals('<a><b/></a>',
+        self.assertEqual(_bytes('<a><b/></a>'),
                           xml_out)
 
         xml_out = self._saxify_serialize(b)
-        self.assertEquals('<b/>',
+        self.assertEqual(_bytes('<b/>'),
                           xml_out)
 
     def test_element_sax_ns(self):
@@ -124,6 +151,30 @@ class ETreeSaxTestCase(HelperTestCase):
         self.assertEqual('{blaA}c',
                          root[1].tag)
 
+    def test_etree_sax_handler_default_ns_None(self):
+        handler = sax.ElementTreeContentHandler()
+        handler.startDocument()
+        handler.startPrefixMapping(None, 'blaA')
+        handler.startElementNS((None, 'a'), 'a', {})
+        handler.startPrefixMapping(None, 'blaB')
+        handler.startElementNS((None, 'b'), 'b', {})
+        handler.endElementNS(  (None, 'b'), 'b')
+        handler.endPrefixMapping(None)
+        handler.startElementNS((None, 'c'), 'c', {})
+        handler.endElementNS(  (None, 'c'), 'c')
+        handler.endElementNS(  (None, 'a'), 'a')
+        handler.endPrefixMapping(None)
+        handler.endDocument()
+
+        new_tree = handler.etree
+        root = new_tree.getroot()
+        self.assertEqual('{blaA}a',
+                         root.tag)
+        self.assertEqual('{blaB}b',
+                         root[0].tag)
+        self.assertEqual('{blaA}c',
+                         root[1].tag)
+
     def test_etree_sax_redefine_ns(self):
         handler = sax.ElementTreeContentHandler()
         handler.startDocument()
@@ -185,17 +236,17 @@ class ETreeSaxTestCase(HelperTestCase):
         
     def _saxify_serialize(self, tree):
         new_tree = self._saxify_unsaxify(tree)
-        f = StringIO()
+        f = BytesIO()
         new_tree.write(f)
-        return f.getvalue()
+        return f.getvalue().replace(_bytes('\n'), _bytes(''))
 
     
 def test_suite():
     suite = unittest.TestSuite()
     suite.addTests([unittest.makeSuite(ETreeSaxTestCase)])
     suite.addTests(
-        [doctest.DocFileSuite('../../../doc/sax.txt')])
+        [make_doctest('../../../doc/sax.txt')])
     return suite
 
 if __name__ == '__main__':
-    unittest.main()
+    print('to test use test.py %s' % __file__)
